#!/usr/bin/env bash

# Directory for build result
today=$(date "+%Y-%m-%d")
 
daily_build_dir=/var/www/html/nntrainer/ci/daily-build
build_result_dir=$daily_build_dir/build_result/$today
build_log_dir=$build_result_dir/log
badge_dir=$daily_build_dir/badge
pack_bin_path=$build_result_dir/bin
gbs_root_path=$build_result_dir/GBS-ROOT
github_io_path=$daily_build_dir/nntrainer.github.io
 
source ../taos/common/api_collection.sh
source ../taos/config/config-environment.sh
source /etc/environment
 
mkdir -p $build_result_dir
mkdir -p $build_log_dir
mkdir -p $pack_bin_path
mkdir -p $gbs_root_path
mkdir -p $daily_build_dir/build_result/latest/
 
# Start daily build
pushd $build_result_dir
 
# Clone NNTrainer
git clone https://github.com/nnstreamer/nntrainer.git nntrainer-temp
nntrainer_root=$build_result_dir/nntrainer-temp
pushd nntrainer-temp
 
global_build_result="pass"
ubuntu_build_result="pass" 
echo -e "[DEBUG] Start daily build and unit test : $today"
echo -e "[DEBUG] build_root : $nntrainer_root"

echo -e "source /etc/environment"
source /etc/environment
 
# check if dependent packages are installed
# the required packages are gbs.
check_cmd_dep sudo
check_cmd_dep curl
check_cmd_dep gbs 
check_cmd_dep tee 
check_cmd_dep pdebuild
check_cmd_dep debootstrap
check_cmd_dep debuild

# ubuntu build
# Build a package with pdebuild 
echo -e "[DEBUG] Starting 'pdebuild --use-pdebuild-internal ...' command."
echo -e "[DEBUG] The pdebuild start on $today"
echo -e "[DEBUG] current working directory : $(pwd)" 
 
time pdebuild 2> ${build_log_dir}/build_log_ubuntu_error_${today}.txt 1> ${build_log_dir}/build_log_ubuntu_output_${today}.txt
if [[ $? -ne 0 ]]; then           
    echo "[DEBUG] Oooops!!!!!! pdebuild failed with compiler optimization"
    ubuntu_build_result="fail" 
else
    echo "[DEBUG] Successfully pdebuild is passed with compiler optimization"
fi   
echo -e "[DEBUG] The pdebuild with compiler optimization finished"

sed -i "s/=release/=debug/" meson.build

time pdebuild 2>> ${build_log_dir}/build_log_ubuntu_error_${today}.txt 1>> ${build_log_dir}/build_log_ubuntu_output_${today}.txt
if [[ $? -ne 0 ]]; then
    echo "[DEBUG] Oooops!!!!!! pdebuild failed without compiler optimization"
    ubuntu_build_result="fail" 
else
    echo "[DEBUG] Successfully pdebuild is passed without compiler optimization"
fi   
echo -e "[DEBUG] The pdebuild without compiler optimization finished"

sed -i "s/=debug/=release/" meson.build

# If the debian files exists, let's remove these files.
debfiles=(../*.dsc)
mkdir -p $pack_bin_path/bin/ubuntu
if [[ -f ${debfiles[0]} ]]; then
    echo -e "Removing unnecessary debian files..."
    echo -e "The binary files will be temporarily archived in /var/cache/pbuilder/ folder."
    sudo mv ../*.deb $pack_bin_path
    sudo rm -rf ../*.tar.gz ../*.dsc ../*.changes ../*.build
    if [[ $? -ne 0 ]]; then
        echo -e "[DEBUG][FAILED] Ubuntu/pdebuild: Oooops!!!!! Unnecessary files are not removed."
    else
        echo -e "[DEBUG][PASSED] Ubuntu/pdebuild: It is okay. Unnecessary files are successfully removed."
    fi
fi      

# Apptest                         
# Check if nntrainer-based sample applications can be run.
# bash ${daily_build_dir}/app_test.sh $nntrainer_root >> ${build_log_dir}/build_log_apptest_${today}.txt
# if [[ $? -ne 0 ]]; then           
#     echo "[DEBUG] Oooops!!!!!! apptest is failed"
#     ubuntu_build_result="fail" 
# else
#     echo "[DEBUG] Successfully apptest is passed"
# fi

if [[ ${ubuntu_build_result} == "pass" ]]; then
    echo -e "[DEBUG][PASSED] Successfully pde build is passed. Return value is ($result)."
    python $badge_dir/gen_badge.py test pass brightgreen $badge_dir/ubuntu_result_badge.svg
else
    echo -e "[DEBUG][FAILED] Oooops!!!!!! pde build is failed. Return value is ($result)."
    python $badge_dir/gen_badge.py test fail critical $badge_dir/ubuntu_result_badge.svg 
    global_build_result="fail"
fi

echo -e "[DEBUG] current folder is $(pwd)."
daily_build_android_type="arm64-v8a, armeabi-v7a"

android_build_result="fail"

for build_type in $daily_build_android_type
do

    # NNTrainer root directory for build.
    export NNSTRAINER_ROOT=$(pwd)
    # Options:
    # a. TODO: A trigger option is to be used as PR number and PR time (a trick)
    #          to support Out-of-PR (OOP) killer.
    # b. TODO: If you meet a privilege issue, you need to execute a local test with
    #          "sudo -Hu www-data ndk-build" statement.
    echo -e "[DEBUG] Starting 'ndk-build ...' command."
    echo -e "[DEBUG] The ndk-build starts at $(date -R)"
    echo -e "[DEBUG] The current directory: $(pwd)."
    pushd ./jni/
    if [ $build_type == "armeabi-v7a" ]; then
	if [ -f Application.mk ]; then
	    sed -i 's/arm64-v8a/armeabi-v7a/' Application.mk
	fi
    fi
    # Build a nntrainer library (e.g., libnntrainer.so)
    echo -e "[DEBUG] Compiling  a nntrainer library: $(pwd)."
    echo -e "time ndk-build NDK_PROJECT_PATH=./ APP_BUILD_SCRIPT=./Android.mk NDK_APPLICATION_MK=./Application.mk -j$(nproc)"
    time /var/www/html/android/android-ndk-r20b/ndk-build APP_BUILD_SCRIPT=./Android.mk NDK_APPLICATION_MK=./Application.mk -j$(nproc) \
         2>> ${build_log_dir}/build_log_android_error_${today}.txt \
         1>> ${build_log_dir}/build_log_android_output_${today}.txt
    result=$(($result+$?))
    echo -e "nntrainer result : ${result}"
    mkdir -p ${pack_bin_path}/android/nntrainer
    cp -r ../libs ${pack_bin_path}/android/nntrainer/
    popd

    if [ -d "./Applications/utils/jni" ]; then
	pushd ./Applications/utils/jni/
	if [ $build_type == "armeabi-v7a" ]; then
	    if [ -f Application.mk ]; then
		sed -i 's/arm64-v8a/armeabi-v7a/' Application.mk
	    fi
	fi	
	echo -e "[DEBUG] Compiling  a nntrainer utils library: $(pwd)."
	echo -e "time ndk-build NDK_PROJECT_PATH=./ APP_BUILD_SCRIPT=./Android.mk NDK_APPLICATION_MK=./Application.mk -j$(nproc)"
	time /var/www/html/android/android-ndk-r20b/ndk-build APP_BUILD_SCRIPT=./Android.mk NDK_APPLICATION_MK=./Application.mk -j$(nproc) \
	     2>> ${build_log_dir}/build_log_android_error_${today}.txt \
	     1>> ${build_log_dir}/build_log_android_output_${today}.txt
	result=$(($result+$?))
	echo -e "app_utils result : ${result}"
	mkdir -p ${pack_bin_path}/android/Applications/utils
	cp -r ../libs ${pack_bin_path}/android/Applications/utils
	popd

	# Build a test application
	echo -e "[DEBUG] Compiling  a nntrainer-based application: $(pwd)."
	APPLICATION_LIST=`find Applications -mindepth 1 -maxdepth 1 -type d -not -name "*utils*"`
	if [[ $? != 0 ]]; then
            echo "[DEBUG] Oooops. Please check $APPLICAITON_PATH in configuraton file is vaild or not."
	fi

	for i in ${APPLICATION_LIST}; do
	    jni_paths=`find $i -type d -name jni`
	    if [ -z "$jni_paths" ]; then
		continue
	    fi
	    for jni in ${jni_paths}; do 
		echo "[DEBUG] Compile Application: ${jni}."
		pushd ${jni}
		if [ $build_type == "armeabi-v7a" ]; then
		    if [ -f Application.mk ]; then
			sed -i 's/arm64-v8a/armeabi-v7a/' Application.mk
		    fi
		fi	
		echo -e "time ndk-build NDK_PROJECT_PATH=./ APP_BUILD_SCRIPT=./Android.mk NDK_APPLICATION_MK=./Application.mk -j$(nproc)"
		time /var/www/html/android/android-ndk-r20b/ndk-build APP_BUILD_SCRIPT=./Android.mk NDK_APPLICATION_MK=./Application.mk -j$(nproc) \
		     2>> ${build_log_dir}/build_log_android_error_${today}.txt \
		     1>> ${build_log_dir}/build_log_android_output_${today}.txt
		result=$(($result+$?))
		echo -e "${jni_path} result : ${result}"
		mkdir -p ${pack_bin_path}/android/${i}
		cp -r ../libs ${pack_bin_path}/android/${i}
		popd
	    done
	done
	if [[ $result -eq 0 ]]; then
	    android_build_result="pass"
	fi

	if [[ ${android_build_result} == "pass" ]]; then
	    echo -e "[DEBUG][PASSED] Successfully android build is passed. Return value is ($result)."
	    python $badge_dir/gen_badge.py test pass brightgreen $badge_dir/${build_type}_android_result_badge.svg
	fi
    else
	echo -e "[DEBUG][FAILED] Oooops!!!!!! android build is failed. Return value is ($result)."
	python $badge_dir/gen_badge.py test fail critical $badge_dir/${build_type}_andoird_result_badge.svg 
	global_build_result="fail"
    fi
done

# Tizen build
daily_build_arch_type="x86_64 armv7l aarch64 i586"
#daily_build_arch_type="" #to skip tizen test

for arch in $daily_build_arch_type
do
    echo "[DEBUG] gbs build for $arch"
    if [ $arch == "x86_64" ]; then
	gbs build -A $arch --define "_skip_debug_rpm 1" --define "unit_test 1" --buildroot $gbs_root_path 2> ${build_log_dir}/build_log_tizen_${arch}_error_${today}.txt 1> ${build_log_dir}/build_log_tizen_${arch}_output_${today}.txt --clean
    else
	gbs build -A $arch --define "_skip_debug_rpm 1" --buildroot $gbs_root_path 2> ${build_log_dir}/build_log_tizen_${arch}_error_${today}.txt 1> ${build_log_dir}/build_log_tizen_${arch}_output_${today}.txt --clean	
    fi

    result=$?
    echo "[DEBUG] gbs build finished"
    # Let's check if build procedure is normally done.
    if [[ $result -eq 0 ]]; then
        echo -e "[DEBUG][PASSED] Successfully build checker is passed. Return value is ($result)."
        python $badge_dir/gen_badge.py test pass brightgreen $badge_dir/${arch}_result_badge.svg 
    else
        echo -e "[DEBUG][FAILED] Oooops!!!!!! build checker is failed. Return value is ($result)."
        python $badge_dir/gen_badge.py test fail critical $badge_dir/${arch}_result_badge.svg        
        global_build_result="fail"
    fi
    
    # If the ./GBS-ROOT/ folder exists, let's remove this folder.
    # Note that this folder consume too many storage space (on average 9GiB).
    if [[ -d $gbs_root_path ]]; then
        echo "[DEBUG] GBS-ROOT folder is exist!"
        echo "Archiving .rpm files for ${arch} ..."
        ls -al  $gbs_root_path/local/repos/tizen/${arch}/RPMS/
        sudo cp -arf $gbs_root_path/local/repos/tizen/${arch}/RPMS $pack_bin_path || echo -e "[DEBUG] Can't copy .rpm files."
	
        if [[ $? -eq 0 ]]; then
            echo "[DEBUG] [PASSED] Successfully copy rpm files !"
        else
            echo "[DEBUG] [FAILED] Faild to copu rpm files.. "
	fi
	
        echo "Removing GBS-ROOT folder..."
        sudo rm -rf $gbs_root_path
	
        if [[ $? -ne 0 ]]; then
            echo "[DEBUG][FAILED] Tizen/gbs: Oooops!! GBS-ROOT folder is not removed."
        else
            echo "[DEBUG][PASSED] Tizen/gbs: GBS-ROOT folder is successfully removed."
        fi
    else
        echo "[DEBUG] GBS-ROOT folder is not exist..."
    fi
done

rm -rf $daily_build_dir/build_result/latest/* 
# Generate NNTrainer documents and push to nntrainer.github.io
echo "[DEBUG] Generate NNTrainer documents"
export PATH="/var/www/html/.local/bin:$PATH"
./docs/hotdoc/gen-hotdoc.sh

cp -r ./docs/NNTrainer-doc/html/* ${github_io_path}/
pushd ${github_io_path}
source ../../taos/config/config-environment.sh
git add *                        
git commit -s -m "$today : Update nntrainer.github.io"
git push https://${GITHUB_ACCOUNT}:${TOKEN}@github.com/nntrainer/nntrainer.github.io.git main -f
popd

rm -rf $nntrainer_root
 
if [[ "$global_build_result" == "pass" ]]; then
    echo "[DEBUG] Successfully all build and unit test are passed"
    python $badge_dir/gen_badge.py "daily build" pass brightgreen $badge_dir/daily_build_badge.svg
else                                      
    echo "[DEBUG] Oooops. One of the build or unit test is failed.."
    python $badge_dir/gen_badge.py "daily build" fail critical $badge_dir/daily_build_badge.svg
fi  

tar -zcvf $build_result_dir/$today.tar.gz $build_result_dir
cp $build_result_dir/$today.tar.gz $daily_build_dir/build_result/latest/

echo "https://nntrainer.github.io" > $daily_build_dir/build_result/latest/nntrainer_documentation
